@@ -119,6 +119,8 @@ func (cache *Cache) Stats() Statistics {
 // MemoryUsage returns the current memory usage of the cache's dataset in bytes
 // If MaxMemoryUsage is set to NoMaxMemoryUsage, this will return 0
 func (cache *Cache) MemoryUsage() int {
+	cache.mutex.RLock()
+	defer cache.mutex.RUnlock()
 	return cache.memoryUsage
 }
 
@@ -257,10 +259,10 @@ func (cache *Cache) SetWithTTL(key string, value any, ttl time.Duration) {
 		// Cache entry doesn't exist, so we have to create a new one
 		entry = &Entry{
 			Key:               key,
-			Value:             value,
 			RelevantTimestamp: time.Now(),
 			next:              cache.head,
 		}
+		entry.Value = value
 		if cache.head == nil {
 			cache.tail = entry
 		} else {
@@ -283,7 +285,6 @@ func (cache *Cache) SetWithTTL(key string, value any, ttl time.Duration) {
 			// Subtract the old entry from the cache's memoryUsage
 			cache.memoryUsage -= entry.SizeInBytes()
 		}
-		// Update existing entry's value
 		entry.Value = value
 		entry.RelevantTimestamp = time.Now()
 		if cache.maxMemoryUsage != NoMaxMemoryUsage {
@@ -514,8 +515,10 @@ func (cache *Cache) TTL(key string) (time.Duration, error) {
 //
 // Returns true if the cache key exists and has had its expiration time altered
 func (cache *Cache) Expire(key string, ttl time.Duration) bool {
+	cache.mutex.Lock()
 	entry, ok := cache.get(key)
 	if !ok || entry.Expired() {
+		cache.mutex.Unlock()
 		return false
 	}
 	if ttl != NoExpiration {
@@ -523,6 +526,7 @@ func (cache *Cache) Expire(key string, ttl time.Duration) bool {
 	} else {
 		entry.Expiration = NoExpiration
 	}
+	cache.mutex.Unlock()
 	return true
 }
 
